@@ -1,7 +1,8 @@
 package IO::Pager::Unbuffered;
-our $VERSION = 0.30;
+our $VERSION = 10.02;
 
 use strict;
+use warnings;
 use base qw( IO::Pager );
 use SelectSaver;
 
@@ -16,7 +17,8 @@ sub new(;$) {  # [FH], procedural
       return $_[1] if defined($_[2]) && $_[2] eq 'procedural';
 
       #...fall back to IO::Handle for transparent OO programming
-      eval "require IO::Handle" && return new IO::Handle;
+      eval "require IO::Handle" or die $@;
+      return IO::Handle->new_from_fd(fileno($_[1]), 'w');
   }
   $!=$@, return 0 if $@ =~ 'pipe';
 
@@ -39,6 +41,8 @@ sub open(;$) { # [FH]
 
 __END__
 
+=pod
+
 =head1 NAME
 
 IO::Pager::Unbuffered - Pipe output to PAGER if destination is a TTY
@@ -57,14 +61,14 @@ IO::Pager::Unbuffered - Pipe output to PAGER if destination is a TTY
   {
     # You can also use scalar filehandles...
     my $token = IO::Pager::Unbuffered::open($FH) or warn($!);
-    print $FH "No globs or barewords for us thanks!\n";
+    print $FH "No globs or barewords for us thanks!\n" while 1;
   }
 
   {
     # ...or an object interface
     my $token = new IO::Pager::Unbuffered;
 
-    $token->print("OO shiny...\n");
+    $token->print("OO shiny...\n") while 1;
   }
 
 =head1 DESCRIPTION
@@ -84,7 +88,7 @@ All methods are inherited from IO::Pager; except for instantiation.
 You probably want to do something with SIGPIPE eg;
 
   eval {
-    $SIG{PIPE} = sub { die };
+    local $SIG{PIPE} = sub { die };
     local $STDOUT = IO::Pager::open(*STDOUT);
 
     while (1) {
@@ -110,7 +114,7 @@ Significant proddage provided by Tye McQueen.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2003-2012 Jerrad Pierce
+Copyright (C) 2003-2018 Jerrad Pierce
 
 =over
 
